import type { GeneratedSchemaObject } from '../Schema';
import type { Selection } from '../Selection';
import { CacheNormalizationHandler } from './normalization';
import { CacheSnapshot } from './persistence';
export type CacheRoot = {
    query?: CacheObject;
    mutation?: CacheObject;
    subscription?: CacheObject;
};
export type CacheNode = CacheLeaf | CacheNode[] | CacheObject;
export type CacheObject = {
    __typename?: string;
    [key: string]: CacheNode | CacheLeaf;
};
export type CacheLeaf = string | number | boolean | null | undefined;
export type CacheOptions = {
    readonly maxAge?: number;
    /**
     * `false` to disable normalized cache, which usually means manual
     * refetching after mutations.
     */
    readonly normalization?: boolean | CacheNormalizationHandler;
    readonly staleWhileRevalidate?: number;
};
export type CacheDataContainer<TData extends CacheNode = CacheNode> = {
    data: TData;
    /**
     * data is open to TTL eviction after this time, before that only LRU eviction
     * may take place.
     */
    expiresAt: number;
    /**
     * A hint for clients to revaliate, does not affect automatic cache eviction.
     */
    swrBefore?: number;
    /**
     * Internal reference of cache eviction timeout, subject to change in the
     * future.
     */
    timeout?: ReturnType<typeof setTimeout>;
};
export type CacheListener = <TData extends CacheNode = CacheNode>(value: TData) => void;
export type CacheGetOptions = {
    includeExpired?: boolean;
};
export type CacheSetOptions = {
    skipNotify?: boolean;
};
/**
 * A scoped cache for accessors, selections and data with expiry awareness.
 */
export declare class Cache {
    #private;
    get maxAge(): number;
    get staleWhileRevalidate(): number;
    get normalizationOptions(): CacheNormalizationHandler | undefined;
    constructor(data?: CacheSnapshot, { maxAge, staleWhileRevalidate, normalization, }?: CacheOptions);
    restore(data: CacheSnapshot): void;
    /** Subscribe to cache changes. */
    subscribe(paths: string[], fn: CacheListener): () => void;
    getAccessor<TSchemaType extends GeneratedSchemaObject>(selection: Selection, createAccessor: () => TSchemaType): TSchemaType;
    /**
     * Retrieve cache values by first 2 path segments, e.g. `query.todos` or
     * `mutation.createTodo`.
     */
    get(path: string, { includeExpired }?: CacheGetOptions): CacheDataContainer | undefined;
    /**
     * Merge objects into the current cache, recursively normalize incoming values
     * if normalization is enabled. Notifies cache listeners afterwards.
     *
     * Example value: `{ query: { foo: "bar" } }`
     */
    set(values: CacheRoot, { skipNotify }?: CacheSetOptions): void;
    toJSON(): any;
}
