'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const graphqlWs = require('graphql-ws');
const ws = require('ws');
const query = require('../Cache/query.js');
const index$1 = require('../Error/index.js');
const useMetaStateHack = require('../Helpers/useMetaStateHack.js');
const index = require('../QueryBuilder/index.js');
const retry = require('../Error/retry.js');

const fetchSelections = (selections, {
  cache,
  debugger: debug,
  fetchOptions,
  operationName
}) => Promise.all(
  index.buildQuery(selections, operationName).map(
    async ({
      query: query$1,
      variables,
      operationName: operationName2,
      extensions: { type, hash } = {}
    }) => {
      if (!type)
        throw new index$1.GQtyError(`Invalid query type: ${type}`);
      if (!hash)
        throw new index$1.GQtyError(`Expected query hash.`);
      const queryPayload = {
        query: query$1,
        variables,
        operationName: operationName2,
        extensions: { type, hash }
      };
      const { data, errors, extensions } = await query.dedupePromise(
        cache,
        hash,
        type === "subscription" ? () => doSubscribeOnce(queryPayload, fetchOptions) : () => doFetch(queryPayload, { ...fetchOptions, selections })
      );
      const result = {
        data,
        extensions: { ...extensions, ...queryPayload.extensions }
      };
      if (errors == null ? void 0 : errors.length) {
        result.error = index$1.GQtyError.fromGraphQLErrors(errors);
      }
      debug == null ? void 0 : debug.dispatch({
        cache,
        request: queryPayload,
        result,
        selections
      });
      return result;
    }
  )
);
const subsRef = /* @__PURE__ */ new WeakMap();
const subscribeSelections = (selections, fn, {
  cache,
  debugger: debug,
  fetchOptions: { subscriber },
  operationName,
  onSubscribe,
  onComplete
}) => {
  const unsubscribers = /* @__PURE__ */ new Set();
  Promise.all(
    index.buildQuery(selections, operationName).map(
      ({
        query: query$1,
        variables,
        operationName: operationName2,
        extensions: { type, hash } = {}
      }) => {
        if (!type)
          throw new index$1.GQtyError(`Invalid query type: ${type}`);
        if (!hash)
          throw new index$1.GQtyError(`Expected query hash.`);
        if (type === "subscription" && !subscriber) {
          throw new index$1.GQtyError(`Missing subscriber for subscriptions.`);
        }
        const queryPayload = {
          query: query$1,
          variables,
          operationName: operationName2,
          extensions: { type, hash }
        };
        let subscriptionId;
        if (isWsClient(subscriber)) {
          {
            const unsub = subscriber == null ? void 0 : subscriber.on("message", (message) => {
              switch (message.type) {
                case graphqlWs.MessageType.ConnectionAck: {
                  unsub == null ? void 0 : unsub();
                  onSubscribe == null ? void 0 : onSubscribe();
                  break;
                }
              }
            });
          }
          {
            const unsub = subscriber == null ? void 0 : subscriber.on("message", (message) => {
              var _a;
              switch (message.type) {
                case graphqlWs.MessageType.ConnectionAck: {
                  break;
                }
                case graphqlWs.MessageType.Subscribe: {
                  if (((_a = message.payload.extensions) == null ? void 0 : _a.hash) !== hash)
                    return;
                  subscriptionId = message.id;
                  debug == null ? void 0 : debug.dispatch({
                    cache,
                    label: `[id=${subscriptionId}] [create]`,
                    request: queryPayload,
                    selections
                  });
                  unsub == null ? void 0 : unsub();
                  break;
                }
              }
            });
          }
        } else if (isSseClient(subscriber)) {
          subscriptionId = "EventSource";
          onSubscribe == null ? void 0 : onSubscribe();
        } else if (type === "subscription") {
          throw new index$1.GQtyError(`Please specify a subscriber for subscriptions.`);
        }
        const next = ({ data, errors, extensions }) => {
          if (data === void 0)
            return;
          const result = {
            data,
            extensions: { ...extensions, ...queryPayload.extensions }
          };
          if (errors == null ? void 0 : errors.length) {
            result.error = index$1.GQtyError.fromGraphQLErrors(errors);
          }
          debug == null ? void 0 : debug.dispatch({
            cache,
            label: subscriptionId ? `[id=${subscriptionId}] [data]` : void 0,
            request: queryPayload,
            result,
            selections
          });
          fn(result);
        };
        const error = (error2) => {
          if (error2 == null) {
            throw new index$1.GQtyError(`Subscription sink closed without an error.`);
          }
          debug == null ? void 0 : debug.dispatch({
            cache,
            label: subscriptionId ? `[id=${subscriptionId}] [error]` : void 0,
            request: queryPayload,
            selections
          });
          fn({ error: error2, extensions: queryPayload.extensions });
        };
        let dispose;
        const promise = query.dedupePromise(cache, hash, () => {
          return new Promise((complete) => {
            dispose = subscriber.subscribe(
              queryPayload,
              {
                next,
                error(err) {
                  if (Array.isArray(err)) {
                    error(index$1.GQtyError.fromGraphQLErrors(err));
                  } else if (!isCloseEvent(err)) {
                    error(index$1.GQtyError.create(err));
                  }
                  this.complete();
                },
                complete() {
                  debug == null ? void 0 : debug.dispatch({
                    cache,
                    label: subscriptionId ? `[id=${subscriptionId}] [unsubscribe]` : void 0,
                    request: queryPayload,
                    selections
                  });
                  complete();
                }
              }
            );
          });
        });
        if (dispose) {
          subsRef.set(promise, { dispose, count: 1 });
        } else if (subsRef.get(promise)) {
          subsRef.get(promise).count++;
        }
        unsubscribers.add(() => {
          const ref = subsRef.get(promise);
          if (ref && --ref.count <= 0) {
            setTimeout(() => {
              if (ref.count <= 0) {
                ref.dispose();
              }
            });
          }
        });
        return promise;
      }
    )
  ).finally(() => onComplete == null ? void 0 : onComplete());
  return () => {
    unsubscribers.forEach((fn2) => fn2());
  };
};
const doFetch = async (payload, {
  fetcher,
  retryPolicy,
  selections,
  ...fetchOptions
}) => {
  const doDoFetch = () => fetcher(payload, fetchOptions);
  try {
    const promise = doDoFetch();
    useMetaStateHack.notifyFetch(promise, selections);
    return await promise;
  } catch (error) {
    if (!retryPolicy || !(error instanceof Error))
      throw error;
    return new Promise((resolve, reject) => {
      retry.doRetry(retryPolicy, {
        onLastTry: async () => {
          try {
            const promise = doDoFetch();
            useMetaStateHack.notifyRetry(promise, selections);
            resolve(await promise);
          } catch (e) {
            reject(e);
          }
        },
        onRetry: async () => {
          const promise = doDoFetch();
          useMetaStateHack.notifyRetry(promise, selections);
          resolve(await promise);
        }
      });
    });
  }
};
const doSubscribeOnce = async ({ query, variables, operationName }, { subscriber }) => {
  if (!subscriber) {
    throw new index$1.GQtyError(`Subscription client is required for subscritions.`);
  }
  return new Promise(
    (resolve, reject) => {
      let result;
      const unsubscribe = subscriber.subscribe(
        {
          query,
          variables: variables != null ? variables : {},
          operationName: operationName != null ? operationName : void 0
        },
        {
          next(data) {
            result = data;
            unsubscribe();
          },
          error(error) {
            if (isCloseEvent(error)) {
              resolve(result);
            } else if (Array.isArray(error)) {
              reject(index$1.GQtyError.fromGraphQLErrors(error));
            } else {
              reject(error);
            }
          },
          complete() {
            if (!result) {
              throw new index$1.GQtyError(`Subscription completed without data`);
            }
            resolve(result);
          }
        }
      );
    }
  );
};
const isCloseEvent = (input) => {
  const error = input;
  return error.type === "close" && error.target instanceof ws.WebSocket || typeof error.code === "number" && [
    4004,
    4005,
    4400,
    4401,
    4403,
    4406,
    4408,
    4409,
    4429,
    4499,
    4500,
    4504
  ].includes(error.code);
};
const isWsClient = (client) => {
  return client !== void 0 && typeof client.on === "function";
};
const isSseClient = (client) => client !== void 0 && !isWsClient(client);

exports.fetchSelections = fetchSelections;
exports.isCloseEvent = isCloseEvent;
exports.subscribeSelections = subscribeSelections;
