import type { BaseGeneratedSchema, FetchOptions } from '.';
import type { Cache } from '../Cache';
import type { GQtyError, RetryOptions } from '../Error';
import type { ScalarsEnumsHash, Schema } from '../Schema';
import type { Selection } from '../Selection';
import { SchemaContext } from './context';
import type { Debugger } from './debugger';
import { Unsubscribe } from './resolveSelections';
export type CreateResolversOptions = {
    cache: Cache;
    debugger?: Debugger;
    depthLimit: number;
    fetchOptions: FetchOptions;
    scalars: ScalarsEnumsHash;
    schema: Readonly<Schema>;
    parentContext?: SchemaContext;
};
export type Resolvers<TSchema extends BaseGeneratedSchema> = {
    /**
     * Create internal parts for `resolve()` and `subscribe()`, useful for
     * custom fetching logics. The React package uses this funciton.
     */
    createResolver: CreateResolverFn<TSchema>;
    /**
     * Query, mutation and subscription in a promise.
     *
     * Selections to queries and mutations are fetched with
     * `fetchOptions.fetcher`, the result is resolved with the cache updated
     * according to the current fetch policy
     *
     * Subscriptions are disconnected upon delivery of the first data message,
     * the cache is updated and the data is resolved, essentially behaving like
     * a promise.
     */
    resolve: ResolveFn<TSchema>;
    /**
     * Query, mutation and subscription in an async generator.
     *
     * Subscription data continuously update the cache, while queries and
     * mutations are fetched once and then listen to future cache changes
     * from the same selections.
     *
     * This function subscribes to *cache changes*, termination of underlying
     * subscription (WebSocket/EventSource) does not stop this generator.
     *
     * Calling `.return()` does not terminate pending promises, use
     * `onSubscribe()` to acquire the unsubscribe function.
     */
    subscribe: SubscribeFn<TSchema>;
};
export type CreateResolverFn<TSchema extends BaseGeneratedSchema> = (options?: ResolveOptions) => {
    accessor: TSchema;
    context: SchemaContext;
    resolve: () => Promise<unknown>;
    selections: Set<Selection>;
    subscribe: (callbacks?: {
        onComplete?: () => void;
        onError?: (error: Error | GQtyError) => void;
        onNext?: (value: unknown) => void;
    }) => () => void;
};
export type ResolveFn<TSchema extends BaseGeneratedSchema> = <TData extends Record<string, unknown> = Record<string, unknown>>(fn: DataFn<TSchema>, options?: ResolveOptions) => Promise<TData>;
export type SubscribeFn<TSchema extends BaseGeneratedSchema> = <TData extends Record<string, unknown> = Record<string, unknown>>(fn: DataFn<TSchema>, options?: SubscribeOptions) => AsyncGenerator<TData, void, unknown> & {
    unsubscribe: Unsubscribe;
};
export type DataFn<TSchema> = (schema: TSchema) => void;
export type ResolveOptions = {
    /**
     * Awaits resolution it the query results in a fetch. Specify `false` to
     * immediately return the current cache, placeholder data will be returned on
     * a partial or complete cache miss.
     *
     * @default true
     */
    awaitsFetch?: boolean;
    /**
     * Defines how a query should fetch from the cache and network.
     *
     * - `default`: Serves the cached contents when it is fresh, and if they are
     * stale within `staleWhileRevalidate` window, fetches in the background and
     * updates the cache. Or simply fetches on cache stale or cache miss. During
     * SWR, a successful fetch will not notify cache updates. New contents are
     * served on next query.
     * - `no-store`: Always fetch and does not update on response.
     * GQty creates a temporary cache at query-level which immediately expires.
     * - `reload`: Always fetch, updates on response.
     * - `no-cache`: Same as `reload`, for GraphQL does not support conditional
     * requests.
     * - `force-cache`: Serves the cached contents regardless of staleness. It
     * fetches on cache miss or a stale cache, updates cache on response.
     * - `only-if-cached`: Serves the cached contents regardless of staleness,
     * throws a network error on cache miss.
     *
     * _It takes effort to make sure the above stays true for all supported
     * frameworks, please consider sponsoring so we can dedicate even more time on
     * this._
     */
    cachePolicy?: RequestCache;
    retryPolicy?: RetryOptions;
    onFetch?: (fetchPromise: Promise<unknown>) => void;
    onSelect?: NonNullable<SchemaContext['onSelect']>;
    operationName?: string;
};
export type SubscribeOptions = {
    /**
     * Defines how a query should fetch from the cache and network.
     *
     * - `default`: Serves the cached contents when it is fresh, and if they are
     * stale within `staleWhileRevalidate` window, fetches in the background and
     * updates the cache. Or simply fetches on cache stale or cache miss. During
     * SWR, a successful fetch will not notify cache updates. New contents are
     * served on next query.
     * - `no-store`: Always fetch and does not update on response.
     * GQty creates a temporary cache at query-level which immediately expires.
     * - `no-cache`: Always fetch, updates on response.
     * - `force-cache`: Serves the cached contents regardless of staleness. It
     * fetches on cache miss or a stale cache, updates cache on response.
     * - `only-if-cached`: Serves the cached contents regardless of staleness,
     * throws a network error on cache miss.
     *
     * _It takes effort to make sure the above stays true for all supported
     * frameworks, please consider sponsoring so we can dedicate even more time on
     * this._
     */
    cachePolicy?: RequestCache;
    retryPolicy?: RetryOptions;
    /**
     * Intercept errors thrown from the underlying subscription client or query
     * fetcher.
     *
     * If omitted, the `subscribe()` generator throws and closes on the first
     * error, terminating other active subscriptions triggered from the same
     * selections.
     */
    onError?: (error: unknown) => void;
    onSelect?: NonNullable<SchemaContext['onSelect']>;
    /**
     * Called when a subscription is established, receives an unsubscribe
     * function that immediately terminates the async generator and any pending
     * promise.
     */
    onSubscribe?: (unsubscribe: Unsubscribe) => void;
    operationName?: string;
};
export declare const createResolvers: <TSchema extends BaseGeneratedSchema>({ cache: clientCache, debugger: debug, depthLimit, fetchOptions, fetchOptions: { cachePolicy: defaultCachePolicy, retryPolicy: defaultRetryPoliy, }, scalars, schema, parentContext, }: CreateResolversOptions) => Resolvers<TSchema>;
