import { createSchemaAccessor } from '../Accessor/index.mjs';
import { pick } from '../Utils/pick.mjs';
import { createContext } from './context.mjs';
import { fetchSelections, subscribeSelections } from './resolveSelections.mjs';
import { updateCaches } from './updateCaches.mjs';

const asyncItDoneMessage = { done: true };
const createResolvers = ({
  cache: clientCache,
  debugger: debug,
  depthLimit,
  fetchOptions,
  fetchOptions: {
    cachePolicy: defaultCachePolicy = "default",
    retryPolicy: defaultRetryPoliy
  },
  scalars,
  schema,
  parentContext
}) => {
  const createResolver = ({
    cachePolicy = defaultCachePolicy,
    onSelect,
    onSubscribe,
    operationName,
    retryPolicy = defaultRetryPoliy
  } = {}) => {
    const selections = /* @__PURE__ */ new Set();
    const context = createContext({
      cache: clientCache,
      depthLimit,
      cachePolicy,
      onSelect(selection, cache) {
        if (!selections.has(selection)) {
          selections.add(selection);
          parentContext == null ? void 0 : parentContext.onSelect(selection, cache);
        }
        onSelect == null ? void 0 : onSelect(selection, cache);
      },
      scalars,
      schema
    });
    const accessor = createSchemaAccessor(context);
    const resolve = async () => {
      if (!context.shouldFetch)
        return;
      if (cachePolicy === "only-if-cached") {
        throw new TypeError("Failed to fetch");
      }
      return fetchSelections(selections, {
        cache: context.cache,
        debugger: debug,
        fetchOptions: {
          ...fetchOptions,
          cachePolicy,
          retryPolicy
        },
        operationName
      }).then((results) => {
        updateCaches(
          results,
          cachePolicy !== "no-store" && context.cache !== clientCache ? [context.cache, clientCache] : [context.cache],
          { skipNotify: !context.notifyCacheUpdate }
        );
        return results;
      });
    };
    const subscribe = ({
      onComplete,
      onError,
      onNext
    } = {}) => {
      const unsubscibers = /* @__PURE__ */ new Set();
      const unsubscribe = () => {
        for (const unsubscribe2 of unsubscibers) {
          unsubscribe2();
        }
      };
      if (onNext) {
        const unsubscribe2 = context.cache.subscribe(
          [...selections].map((s) => s.cacheKeys.join(".")),
          (data) => onNext({ data })
        );
        unsubscibers.add(unsubscribe2);
      }
      const subscriptionSelections = /* @__PURE__ */ new Set();
      const promises = [];
      if (context.shouldFetch) {
        for (const selection of selections) {
          if (selection.root.key === "subscription") {
            selections.delete(selection);
            subscriptionSelections.add(selection);
          }
        }
        promises.push(resolve());
      }
      for (const selection of subscriptionSelections) {
        selections.add(selection);
      }
      {
        const promise = new Promise((resolve2, reject) => {
          const unsubscribe2 = subscribeSelections(
            subscriptionSelections,
            ({ data, error, extensions }) => {
              if (error) {
                onError == null ? void 0 : onError(error);
                reject(error);
              } else if (data !== void 0) {
                updateCaches(
                  [{ data, error, extensions }],
                  cachePolicy !== "no-store" && context.cache !== clientCache ? [context.cache, clientCache] : [context.cache],
                  { skipNotify: !context.notifyCacheUpdate }
                );
              } else ;
            },
            {
              cache: context.cache,
              debugger: debug,
              fetchOptions: {
                ...fetchOptions,
                cachePolicy,
                retryPolicy
              },
              operationName,
              onSubscribe: () => onSubscribe == null ? void 0 : onSubscribe(unsubscribe2),
              onComplete: () => resolve2()
            }
          );
          unsubscibers.add(unsubscribe2);
        });
        promises.push(promise);
      }
      Promise.allSettled(promises).finally(onComplete);
      return unsubscribe;
    };
    return { accessor, context, resolve, selections, subscribe };
  };
  return {
    createResolver,
    resolve: async (fn, options) => {
      var _a, _b;
      const { accessor, resolve, selections } = createResolver(options);
      fn(accessor);
      const dataFn = () => {
        var _a2;
        return (_a2 = fn(accessor)) != null ? _a2 : pick(accessor, selections);
      };
      const fetchPromise = resolve().then(dataFn);
      if ((_a = options == null ? void 0 : options.awaitsFetch) != null ? _a : true) {
        return fetchPromise;
      }
      (_b = options == null ? void 0 : options.onFetch) == null ? void 0 : _b.call(options, fetchPromise);
      return dataFn();
    },
    subscribe: (fn, { onSubscribe, ...options } = {}) => {
      const { accessor, selections, subscribe } = createResolver({
        ...options,
        onSubscribe: (unsubscribe2) => {
          onSubscribe == null ? void 0 : onSubscribe(() => {
            unsubscribe2();
            done = true;
            deferred == null ? void 0 : deferred.resolve();
          });
        }
      });
      fn(accessor);
      const unsubscribe = subscribe({
        onError: (error) => {
          rejected = error;
          deferred == null ? void 0 : deferred.reject(error);
        },
        onNext(value) {
          var _a;
          pending.push((_a = fn(accessor)) != null ? _a : value);
          deferred == null ? void 0 : deferred.resolve();
        }
      });
      let deferred;
      let rejected;
      let done = false;
      const pending = [];
      if (selections.size === 0) {
        done = true;
      }
      return {
        async next() {
          if (rejected !== void 0) {
            throw rejected;
          } else if (done) {
            return asyncItDoneMessage;
          } else if (pending.length > 0) {
            return { value: pending.shift() };
          } else {
            await new Promise((resolve, reject) => {
              deferred = { resolve, reject };
            });
            return this.next();
          }
        },
        async throw(error) {
          throw error;
        },
        async return() {
          unsubscribe();
          return asyncItDoneMessage;
        },
        [Symbol.asyncIterator]() {
          return this;
        },
        unsubscribe
      };
    }
  };
};

export { createResolvers };
